/* GStreamer
 * Copyright (C) 2002, Iain Holmes <iain@prettypeople.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef __GST_CRMOSAIC_H__
#define __GST_CRMOSAIC_H__

#include <gst/gst.h>
#include "nvbufsurface.h"
#include "nvbufsurftransform.h"

#define VERSION "2.01"
#define GST_LICENSE "Proprietary"
#define PACKAGE "crmosaic"
#define GST_PACKAGE_NAME "crmosaic plugin"
#define GST_PACKAGE_ORIGIN "http://www.cr.com/"

G_BEGIN_DECLS
#define GST_TYPE_CRMOSAIC (gst_crmosaic_get_type())
#define GST_CRMOSAIC(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_CRMOSAIC, CRMOSAIC))
#define GST_CRMOSAIC_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_CRMOSAIC, CRMOSAICClass))
#define GST_CRMOSAIC_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_CRMOSAIC, CRMOSAICClass))
#define GST_IS_CRMOSAIC(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_CRMOSAIC))
#define GST_IS_CRMOSAIC_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_CRMOSAIC))
typedef struct _Gstcrmosaic CRMOSAIC;
typedef struct _GstcrmosaicClass CRMOSAICClass;

struct _Gstcrmosaic
{
	GstElement element;

	  // Amount of objects processed in single call to algorithm
	  guint batch_size;
	  // GPU ID on which we expect to execute the task
	  guint gpu_id;
	  // composite buf
	  NvBufSurface *inter_buf;
  	  NvBufSurfTransformCompositeParams composite_params;
	  
	  NvBufSurface *surface[2];
	  gint main_sub;
	  gint mosaic_width;
	  gint mosaic_height;
	  gint mosaic_top;
	  gint mosaic_left;
	  gboolean active;
	  gint counter;

	/* pad */
	GstPad *srcpad;

	GstPad *last_pad; /* protected by object lock */

	GstClockTime tsmp;
	GstClockTime duration;
	GstClockTime last_stop;
	gboolean send_stream_start;

	//////////
	guint mtusize;	// limited by udpsink or spisink
	GMutex lock;

};

struct _GstcrmosaicClass
{
  GstElementClass parent_class;

  gboolean (*src_event) (CRMOSAIC *crmosaic, GstEvent *event);
};

GType gst_crmosaic_get_type (void);

G_END_DECLS

#endif /* __GST_CRMOSAIC_H__ */
